<?php
namespace App\Controller;

use App\Templates;
use App\Exception;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use SimpleXMLElement;

class WebService {
    
    private $util;
    private $integrationService;
    
    private $basePath;
    private $ordersPath;        
    private $processedPath;     
    private $errPath;
    
    private $mailList;
    private $serverName;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util,
                                 \App\Service\IntegrationService $integrationService) {
        
        $this->connection         = $connection;
        $this->util               = $util;
        $this->integrationService = $integrationService;
    }
    
    public function welcome (Request $request, Response $response) {
        
        $sql         = "SELECT CDFILIAL FROM FILIAL";
        $result      = $this->connection->fetchAll($sql);
        $versionInfo = $this->getVersionInfo();
        
        $data = [
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => str_replace('/', '-', $versionInfo['date']),
            'branchs'     => count($result)
        ];
        
        $return = $this->returnStructure('false', '' , $data);
        
        
        $response = $response->withHeader('Content-Type', "application/xml");
        $response->getBody()->write($return);
        
        return $response;
    }
    
    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    private function getClientName() {
        global $CLIENT_NAME;
        return $CLIENT_NAME;
    }
    
    public function getBalance(Request $request, Response $response) {
        try {
            $params   = $request->getParsedBody();
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            if (isset($input['data']['clientCode'])) {
                $clientCode = $input['data']['clientCode'];
            } else {
                throw new Exception\ParamsException("Código de cliente não informado.");
            }
            
            if (isset($input['data']['consumerCode'])) {
                $consumerCode = $input['data']['consumerCode'];
            } else {
                throw new Exception\ParamsException("Código de consumidor não informado.");
            }
            
            if (isset($input['data']['campaignCode'])) {
                $campaignCode = $input['data']['campaignCode'];
            } else {
                throw new Exception\ParamsException("Código de campanha não informado.");
            }
            
            $balance = $this->integrationService->getBalance($clientCode, $consumerCode, $campaignCode);
            
            $return = $this->returnStructure('false', '' , $balance);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $message = 'Erro ao consultar saldo. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);;
        }
    }
    
    public function creditBalance(Request $request, Response $response) {
        global $CLIENT_NAME;
        
        $params   = $request->getParsedBody();
        
        $this->connection->beginTransaction();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
             if (isset($input['data']['subsidiaryCode']) && !empty($input['data']['subsidiaryCode'])) {
                $subsidiaryCode = $input['data']['subsidiaryCode'];
            } else {
                throw new Exception\ParamsException("Código da Filial não informado.");
            }
            
            if (isset($input['data']['clientCode'])) {
                $clientCode = $input['data']['clientCode'];
            } else {
                throw new Exception\ParamsException("Código do Cliente não informado.");
            }
            
            if (isset($input['data']['consumerCode'])) {
                $consumerCode = $input['data']['consumerCode'];
            } else {
                throw new Exception\ParamsException("Código do Consumidor não informado.");
            }
            
            if (isset($input['data']['phone'])) {
                $phone = $input['data']['phone'];
            } else {
                throw new Exception\ParamsException("Telefone do consumidor não informado.");
            }
            
            if (isset($input['data']['posCode'])) {
                $posCode = $input['data']['posCode'];
            } else {
                throw new Exception\ParamsException("Caixa não informado.");
            }
            
            if (isset($input['data']['saleSeq'])) {
                $saleSeq = str_pad($input['data']['saleSeq'], 10, 0, STR_PAD_LEFT);
            } else {
                throw new Exception\ParamsException("saleSeq não informado.");
            }
            
            $branch = $this->integrationService->getBranch($subsidiaryCode);
            
            if(empty($branch))
                throw new Exception\ParamsException("Filial {$subsidiaryCode} não encontrada.");
            
            $checkRegConsumer = $this->integrationService->registerConsumer($subsidiaryCode, $clientCode, $consumerCode, $phone);
            $dtAbertur = isset($input['data']['posOpeningDate']) ? $input['data']['posOpeningDate'] : null;
            $dataProducts  = isset($input['data']['products']) ? $input['data']['products'] : null;

            if(!isset($dataProducts['prod'][0])) {
                $products['prod'][0] = $dataProducts['prod'];
                unset($dataProducts['prod']);
                $dataProducts['prod'] = $products['prod'];
            }

            $score = $this->integrationService->processScore($subsidiaryCode, $posCode, $checkRegConsumer['data'], $dataProducts['prod'], $saleSeq, $dtAbertur);
            
            $data = [
                'consumerName' => $checkRegConsumer['data']['NMCONSUMIDOR'],
                'consumerCode' => $checkRegConsumer['data']['CDCONSUMIDOR'],
                'clientCode'   => $checkRegConsumer['data']['CDCLIENTE'],
                'balance'      => $score
            ];
            
            $return = $this->returnStructure('false', '' , $data);
            
            $this->connection->commit();
            
            $message = $this->formatSMS($branch, $data['consumerName'], $data['balance']['generatedBalance'], $data['balance']['expiration'], 'S');
            $send = $this->sendSMS($phone, $message);
            if(!isset($send['Success']) || !$send['Success'])
                $this->util->writeLog('execution', '', 'Falha no envio de SMS!');
            
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            $message = 'Erro ao creditar saldo. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function redeemBalance(Request $request, Response $response) {
        $params   = $request->getParsedBody();
        
        $this->connection->beginTransaction();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            
            if (isset($input['data']['subsidiaryCode']) && !empty($input['data']['subsidiaryCode'])) {
                $subsidiaryCode = $input['data']['subsidiaryCode'];
            } else {
                throw new Exception\ParamsException("Código da Filial não informado.");
            }
            
            if (isset($input['data']['clientCode'])) {
                $clientCode = $input['data']['clientCode'];
            } else {
                throw new Exception\ParamsException("Código do Cliente não informado.");
            }
            
            if (isset($input['data']['consumerCode'])) {
                $consumerCode = $input['data']['consumerCode'];
            } else {
                throw new Exception\ParamsException("Código do Consumidor não informado.");
            }
            
            if (isset($input['data']['phone'])) {
                $phone = $input['data']['phone'];
            } else {
                throw new Exception\ParamsException("Telefone do consumidor não informado.");
            }
            
            if (isset($input['data']['saleSeq'])) {
                $saleSeq = str_pad($input['data']['saleSeq'], 10, 0, STR_PAD_LEFT);
            } else {
                throw new Exception\ParamsException("saleSeq não informado.");
            }
            
            if (isset($input['data']['posCode'])) {
                $posCode = $input['data']['posCode'];
            } else {
                throw new Exception\ParamsException("Caixa não informado.");
            }
            
            if (isset($input['data']['campaignCode'])) {
                $campaignCode = $input['data']['campaignCode'];
            } else {
                throw new Exception\ParamsException("Código de campanha não informado.");
            }
            
            $branch = $this->integrationService->getBranch($subsidiaryCode);
            
            if(empty($branch))
                throw new Exception\ParamsException("Filial {$subsidiaryCode} não encontrada.");
            
            $balanceUsed = isset($input['data']['balanceUsed']) ?  $this->util->toFloat($input['data']['balanceUsed']) : 0;
            $dtAbertur = isset($input['data']['posOpeningDate']) ? $input['data']['posOpeningDate'] : null;
            
            $consumerDetails = $this->integrationService->getConsumer($clientCode, $consumerCode);
            $balance = $this->integrationService->getBalance($clientCode, $consumerCode, $campaignCode);
            
            if(!$input['data']['balanceUsed'])
                throw new Exception\ParamsException("O valor de resgate solicitado é inválido.");
            
            if($balance['balance'] < $balanceUsed)
                throw new Exception\ParamsException("Saldo insuficiente. Saldo: {$balance['balance']} | Valor de Resgate: {$balanceUsed}");
            
            $this->integrationService->redeemBalance($subsidiaryCode, $posCode, $consumerDetails, $campaignCode, $balanceUsed, $saleSeq, $dtAbertur);
            
            $return = $this->returnStructure('false', 'Resgate de saldo realizado com sucesso.' , []);
            
            $this->connection->commit();
            
            $message = $this->formatSMS($branch, $consumerDetails['NMCONSUMIDOR'], $balanceUsed, null, 'R');
            $send = $this->sendSMS($phone, $message);
            if(!isset($send['Success']) || !$send['Success'])
                $this->util->writeLog('execution', '', 'Falha no envio de SMS!');
                
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            $message = 'Erro ao resgatar saldo. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function cancelBalance(Request $request, Response $response) {
        $params   = $request->getParsedBody();
        
        $this->connection->beginTransaction();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            
            if (isset($input['data']['subsidiaryCode']) && !empty($input['data']['subsidiaryCode'])) {
                $subsidiaryCode = $input['data']['subsidiaryCode'];
            } else {
                throw new Exception\ParamsException("Código da Filial não informado.");
            }
            
            if (isset($input['data']['clientCode'])) {
                $clientCode = $input['data']['clientCode'];
            } else {
                throw new Exception\ParamsException("Código do Cliente não informado.");
            }
            
            if (isset($input['data']['consumerCode'])) {
                $consumerCode = $input['data']['consumerCode'];
            } else {
                throw new Exception\ParamsException("Código do Consumidor não informado.");
            }
            
            if (isset($input['data']['saleSeq'])) {
                $saleSeq = str_pad($input['data']['saleSeq'], 10, 0, STR_PAD_LEFT);
            } else {
                throw new Exception\ParamsException("saleSeq não informado.");
            }
            
            if (isset($input['data']['posCode'])) {
                $posCode = $input['data']['posCode'];
            } else {
                throw new Exception\ParamsException("Caixa não informado.");
            }
            
            
            $branch = $this->integrationService->getBranch($subsidiaryCode);
            
            if(empty($branch))
                throw new Exception\ParamsException("Filial {$subsidiaryCode} não encontrada.");
            
            $consumerDetails = $this->integrationService->getConsumer($clientCode, $consumerCode);

            $this->integrationService->cancelBalance($subsidiaryCode, $posCode, $consumerDetails, $saleSeq);
            
            $return = $this->returnStructure('false', 'Cancelamento realizado com sucesso.' , []);
            
            $this->connection->commit();
            
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            $message = 'Erro ao cancelar credito/resgate de saldo. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function getConsumer(Request $request, Response $response) {
        // Enable CORS for all origins
        header("Access-Control-Allow-Origin: *");
        
        // Allow the following HTTP methods
        header("Access-Control-Allow-Methods: GET, POST, PUT, DELETE, OPTIONS");
        
        // Allow the following headers in the request
        header("Access-Control-Allow-Headers: Content-Type");
        $this->util->writeLog('execution', '', date('d-m-Y H:i:s') . ' - /getConsumer');
        $params   = $request->getParsedBody();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            if (!isset($input['data']['id']) || empty($input['data']['id'])) {
                throw new Exception\ParamsException("Campo id é não informado.");
            }
            
            if (!isset($input['data']['type']) || empty($input['data']['type'])) {
                throw new Exception\ParamsException("Campo type não informado.");
            }
            
            $consumer = $this->integrationService->getConsumerById($input['data']['id'], $input['data']['type']);
            $consumerDetails = $this->integrationService->getConsumer($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);
            
            
            $data = [
                'clientCode'    => $consumerDetails['CDCLIENTE'],
                'consumerCode'  => $consumerDetails['CDCONSUMIDOR'],
                'name'          => $consumerDetails['NMCONSUMIDOR'],
                'email'         => $consumerDetails['DSEMAILCONS'],
                'phone'         => $consumerDetails['NRTELECONS'],
                'cpf'           => $consumerDetails['NRCPFRESPCON'],
                'birthday'      => $consumerDetails['DTNASCCONS'],
                'gender'        => $consumerDetails['IDSEXOCONS'],
                'cep'           => $consumerDetails['NRCEPCONS'],
                'street'        => $consumerDetails['DSENDECONS'],
                'neighborhood'  => $consumerDetails['NMBAIRCONS'],
                'addressNumber' => $consumerDetails['NRENDECONS'],
                'complement'    => $consumerDetails['DSCOMPLENDECONS'],
                'state'         => $consumerDetails['SGESTADO'],
                'city'          => $consumerDetails['NMMUNICIPIO']
            ];
            
            $return = $this->returnStructure('false', '' , $data);
            
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $message = 'Erro. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function updateConsumer(Request $request, Response $response) {
        $this->util->writeLog('execution', '', date('d-m-Y H:i:s') . ' /updateConsumer');
        $params   = $request->getParsedBody();
        
        $this->connection->beginTransaction();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            
            if (!isset($input['data']['clientCode']) || empty($input['data']['clientCode'])) {
                throw new Exception\ParamsException("Codigo de cliente não informado.");
            }
            
            if (!isset($input['data']['consumerCode']) || empty($input['data']['consumerCode'])) {
                throw new Exception\ParamsException("Codigo de consumidor não informado.");
            }
            
            $this->integrationService->updateConsumer($input['data']);
            
            $return = $this->returnStructure('false', 'Registro atualizado com sucesso.' , []);
            
            $this->connection->commit();
            
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            $message = 'Erro na atualização de dados. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }

    private function returnStructure($status, $message, $data) {
        $array = [
            'error'   => $status,
            'message' => $message,
            'data'    => $data
        ];

        $return = $this->buildXmlReturn($array);
        return $return;
    }
    

    private function buildXmlReturn($array) {
        $xml  = new \DOMDocument("1.0", "UTF-8");
        $root = $xml->createElement('root');
      
        $xml->appendChild($root);
        
        function add_element($xml, $parent, $key, $value) {
          $entities = [
              ">"  => "&gt;",
              "<"  => "&lt;",
              "&"  => "&amp;",
              "\"" => "&quot;",
              "'"  => "&apos;"
            ];
          
          if (is_array($value)) {
            $element = $xml->createElement(str_replace(array_keys($entities), array_values($entities), $key));
            $parent->appendChild($element);
            foreach ($value as $subkey => $subvalue) {
                add_element($xml, $element, $subkey, $subvalue);
            }
          } else {
            if (empty($value) && !is_numeric($value)) {
                $element = $xml->createElement($key);
                $element->appendChild($xml->createTextNode(""));
            } else {
                $element = $xml->createElement(str_replace(array_keys($entities), array_values($entities), $key), str_replace(array_keys($entities), array_values($entities), $value));
            }
            $parent->appendChild($element);
          }
        }
        foreach ($array as $key => $value) {
            add_element($xml, $root, $key, $value);
        }
        
        $xml->formatOutput = true;
        return $xml->saveXML();
    }
    
    private function formatSMS($branch, $consumerName, $value = 0, $expiration = null, $type) {
        $consumerName = ucwords(strtolower(substr($consumerName, 0, 15)));
        $parts = explode(' ', strtolower($consumerName));
        $firstName = ucfirst($parts[0]);
        $lastName = isset($parts[1]) ? ' ' . strtoupper(substr($parts[1], 0, 1)) : '';
        
        $consumerName = $firstName.$lastName;
        $consumerName = preg_replace('/[^\p{L}\p{N}\s]/u', '', $consumerName);
        $branchName  = substr($branch['NMFILIAL'], 0, 20);
        
        $today = date('d/m');

        if(!empty($expiration))
            $expiration = date('d/m', strtotime(str_replace("/", "-", $expiration)));
        
        if($type == 'S') {
            $message1 = "{$branchName}: Parabens, {$consumerName}! voce ganhou R$" . $value . " de bonus! Aproveite o desconto na sua proxima compra.";
            $message2 = " Valido de {$today} a {$expiration}.";
            
            $finalMessage = !empty($expiration) ? $message1 . $message2 : $message1;
        }
        
        if($type == 'R') {
            $finalMessage = "{$branchName}: Ola {$consumerName}! Seu resgate de R$" . $value ." de bonus foi concluido com sucesso. Esperamos te ver novamente em breve!";

        }
        
        return $finalMessage;
    }
    
    private function sendSMS($phone, $message) {
        $config  = $this->getClientName(); 
        
        $header = [
            "auth-key: {$config['tokenApi']}",
            "Content-Type: application/json"
        ];
        
        $body = [
            "Receivers" => $phone,
            "Content"   => $message
        ];
        
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL               => $config['urlApiSMS'].'/send',
          CURLOPT_RETURNTRANSFER    => true,
          CURLOPT_ENCODING          => '',
          CURLOPT_MAXREDIRS         => 10,
          CURLOPT_TIMEOUT           => 4,
          CURLOPT_HTTP_VERSION      => CURL_HTTP_VERSION_1_1,
          CURLOPT_SSL_VERIFYPEER    => false,
          CURLOPT_CUSTOMREQUEST     => 'POST',
          CURLOPT_POSTFIELDS        => json_encode($body),
          CURLOPT_HTTPHEADER        => $header,
        ));
        
        $curlResponse = curl_exec($curl);
        $httpCode     = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($curl)) || ($httpCode != 200) || (stripos($curlResponse, "<html>")) || (stripos($curlResponse, "Fatal error"))) {
            $fileName = "REQ-".date("H-i-s") . ".txt";

            $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $requestHeader,
                'url'       => $url,
                'method'    => $method,
                'params'    => $requestBody,
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curlResponse
            ]));
        }
        
        curl_close($curl);
        
        return json_decode($curlResponse, true);
        
    }
}
